use std::{fmt::Display, path::PathBuf};

use crate::lst_parser::error::SyntaxError;

#[derive(Debug)]
pub enum Error {
    Syntax(PathBuf, SyntaxError),
    FailedToOpenFileMod(PathBuf, std::io::Error),
}

impl Error {
    pub fn from_syntax(file: PathBuf, e: SyntaxError) -> Self {
        Self::Syntax(file, e)
    }
}

#[derive(Debug)]
pub enum WorldCreationError {
    FailedToOpenEntryPoint(PathBuf, std::io::Error),
}

impl Display for WorldCreationError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            WorldCreationError::FailedToOpenEntryPoint(entry_path, e) => {
                write!(f, "failed to open entry_point '{entry_path:?}': {e}")
            }
        }
    }
}
